import collections
from bx.intervals import *
from qcmodule import BED

__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


'''
Compare given bed entry to reference gene model
This python3 module was converted from python2.7 code using 2to3
'''

def getCDSExonFromFile(bedfile):
    
    '''Only Extract CDS exon regions from input bed file (must be 12-column).'''        
    ret_lst=[]
    for f in open(bedfile,'r'):
        f = f.strip().split()
        chrom = f[0]
        chrom_start = int(f[1])
        name = f[4]
        strand = f[5]
        cdsStart = int(f[6])
        cdsEnd = int(f[7])
        blockCount = int(f[9])
        blockSizes = [ int(i) for i in f[10].strip(',').split(',') ]
        blockStarts = [ chrom_start + int(i) for i in f[11].strip(',').split(',') ]
          # grab cdsStart - cdsEnd
        cds_exons = []
        cds_seq = ''
        genome_seq_index = []
        
        chrom = chrom + ':' + strand
        for base,offset in zip( blockStarts, blockSizes ):
            if (base + offset) < cdsStart: continue
            if base > cdsEnd: continue
            exon_start = max( base, cdsStart )
            exon_end = min( base+offset, cdsEnd ) 
            #cds_exons.append( (exon_start, exon_end) )
            ret_lst.append([chrom,exon_start,exon_end])
    return ret_lst

def getUTRExonFromFile(bedfile,utr=35):
    '''Only Extract UTR regions from input bed file (must be 12-column). output is 6-column bed format.
    When utr=35 [default], extract both 5' and 3' UTR. When utr=3, only extract 3' UTR. When utr=5,
    only extract 5' UTR'''
    
    ret_lst=[]
    for line in open(bedfile,'r'):
        if line.startswith('#'):continue
        if line.startswith('track'):continue
        if line.startswith('browser'):continue
        fields=line.rstrip('\r\n').split()
        chrom=fields[0]
        strand=fields[5]
        txStart=int(fields[1])
        txEnd=int(fields[2])
        cdsStart=int(fields[6])
        cdsEnd=int(fields[7])       
        exon_start=list(map(int,fields[11].rstrip(',').split(',')))
        exon_start=list(map((lambda x: x + txStart),exon_start))
            
        exon_end=list(map(int,fields[10].rstrip(',').split(',')))
        exon_end=list(map((lambda x,y:x+y),exon_start,exon_end))
               
        chrom = chrom + ':' + strand
        if (utr==35 or utr==5):
            for st,end in zip(exon_start,exon_end):
                if st < cdsStart:
                    utr_st = st
                    utr_end = min(end,cdsStart)
                    ret_lst.append([chrom,utr_st,utr_end])                  
        if (utr==35 or utr==3):
            for st,end in zip(exon_start,exon_end):
                if end > cdsEnd:
                    utr_st = max(st, cdsEnd)
                    utr_end = end
                    ret_lst.append([chrom,utr_st,utr_end]) 
    return ret_lst



def getExonFromFile(bedfile):
    '''Extract ALL exon regions from input bed file (must be 12-column). return list of [chrom:+ st end]'''
    
    ret_lst=[]
    for line in open(bedfile,'r'):
        try:
            if line.startswith('#'):continue
            if line.startswith('track'):continue
            if line.startswith('browser'):continue
            fields=line.rstrip('\r\n').split()
            txStart=int(fields[1])
            chrom=fields[0]
            strand=fields[5]
            geneName=fields[3]
            score=fields[4]
            exon_start=list(map(int,fields[11].rstrip(',').split(',')))
            exon_start=list(map((lambda x: x + txStart),exon_start))
            exon_end=list(map(int,fields[10].rstrip(',').split(',')))
            exon_end=list(map((lambda x,y:x+y),exon_start,exon_end))
        except:
            print("[NOTE:input bed must be 12-column] skipped this line: " + line, end=' ', file=sys.stderr)
            continue
        chrom = chrom + ':' + strand
        for st,end in zip(exon_start,exon_end):
            ret_lst.append([chrom,st,end])
    return ret_lst

def getExonFromFile2(bedfile):
    '''Extract ALL exon regions from input bed file (must be 12-column). return dict'''
    
    ret_dict_full = collections.defaultdict(set)    
    #ret_dict_inner = collections.defaultdict(set)  #trim off start_of_1st_exon and end_of_last_exon
    for line in open(bedfile,'r'):
        tmp=[]
        try:
            if line.startswith('#'):continue
            if line.startswith('track'):continue
            if line.startswith('browser'):continue
            fields=line.rstrip('\r\n').split()
            txStart=int(fields[1])
            chrom=fields[0]
            strand=fields[5]
            geneName=fields[3]
            score=fields[4]
            exon_start=list(map(int,fields[11].rstrip(',').split(',')))
            exon_start=list(map((lambda x: x + txStart),exon_start))
            exon_end=list(map(int,fields[10].rstrip(',').split(',')))
            exon_end=list(map((lambda x,y:x+y),exon_start,exon_end))
            key = chrom + ":" + txstart + "-" + txEnd + ":" + strand + ':' + geneName
        except:
            print("[NOTE:input bed must be 12-column] skipped this line: " + line, end=' ', file=sys.stderr)
            continue
        for st,end in zip(exon_start,exon_end):
            tmp.append(exon_start,exon_end)
        ret_dict_full[key] = set(tmp)
        #ret_dict_inner[key] = set(tmp[1:-1])
    return ret_dict_full



def getUTRExonFromLine(bedline,utr=35):
    '''Extract UTR regions from input bed line. When utr=35 [default], extract both
    5' and 3' UTR. When utr=3, only extract 3' UTR. When utr=5,only extract 5' UTR'''
    
    ret_lst=[]
    line = bedline
    if line.startswith('#'):return None
    if line.startswith('track'):return None
    if line.startswith('browser'):return None
    if not line.strip():return None
    fields=line.rstrip('\r\n').split()
    chrom=fields[0]
    strand=fields[5]
    txStart=int(fields[1])
    txEnd=int(fields[2])
    cdsStart=int(fields[6])
    cdsEnd=int(fields[7])       
    exon_start=list(map(int,fields[11].rstrip(',').split(',')))
    exon_start=list(map((lambda x: x + txStart),exon_start))
        
    exon_end=list(map(int,fields[10].rstrip(',').split(',')))
    exon_end=list(map((lambda x,y:x+y),exon_start,exon_end))
    
    chrom = chromm + ':' + strand
    if (utr==35 or utr==5):
        for st,end in zip(exon_start,exon_end):
            if st < cdsStart:
                utr_st = st
                utr_end = min(end,cdsStart)
                ret_lst.append([chrom,utr_st,utr_end])                  
    if (utr==35 or utr==3):
        for st,end in zip(exon_start,exon_end):
            if end > cdsEnd:
                utr_st = max(st, cdsEnd)
                utr_end = end
                ret_lst.append([chrom,utr_st,utr_end])
    return ret_lst
        

def getCDSExonFromLine(bedline):
    
    '''Extract CDS exon regions from input bed line (must be 12-column).'''     
    ret_lst=[]
    line = bedline
    if line.startswith('#'):return None
    if line.startswith('track'):return None
    if line.startswith('browser'):return None
    if not line.strip():return None
    f = line.strip().split()
    chrom = f[0]
    chrom_start = int(f[1])
    name = f[4]
    strand = f[5]
    cdsStart = int(f[6])
    cdsEnd = int(f[7])
    blockCount = int(f[9])
    blockSizes = [ int(i) for i in f[10].strip(',').split(',') ]
    blockStarts = [ chrom_start + int(i) for i in f[11].strip(',').split(',') ]
        # grab cdsStart - cdsEnd
    cds_exons = []
    cds_seq = ''
    genome_seq_index = []
    chrom = chromm + ':' + strand
    for base,offset in zip( blockStarts, blockSizes ):
        if (base + offset) < cdsStart: continue
        if base > cdsEnd: continue
        exon_start = max( base, cdsStart )
        exon_end = min( base+offset, cdsEnd ) 
        #cds_exons.append( (exon_start, exon_end) )
        ret_lst.append([chrom,exon_start,exon_end]) 
    return ret_lst

def getExonFromLine(bedline):
    '''Extract ALL exon regions from input bed line (must be 12-column). return list of [chrom:+ st end]'''
    
    ret_lst=collections.defaultdict(set)
    line = bedline
    #if line.startswith('#'):continue
    #if line.startswith('track'):continue
    #if line.startswith('browser'):continue
    fields=line.rstrip('\r\n').split()
    txStart=int(fields[1])
    chrom=fields[0]
    strand=fields[5]
    geneName=fields[3]
    score=fields[4]
    exon_start=list(map(int,fields[11].rstrip(',').split(',')))
    exon_start=list(map((lambda x: x + txStart),exon_start))
    exon_end=list(map(int,fields[10].rstrip(',').split(',')))
    exon_end=list(map((lambda x,y:x+y),exon_start,exon_end))
    chrom = chrom + ':' + strand
    for st,end in zip(exon_start,exon_end):
        ret_lst[chrom].add(st)
        ret_lst[chrom].add(end)
    return ret_lst

    
def annotateBed(inputbed,refbed,outfile):
    '''compare inputbed to refbed'''
    ref_exon_ranges = {}
    ref_exon_starts = collections.defaultdict(set)      #key='chrom:+', value=set()
    ref_exon_ends = collections.defaultdict(set)
    OF = open(outfile,'w')

    #read reference bed file
    #Extract CDS exons from reference bed
    tmp = getCDSExonFromFile(refbed)
    for i in tmp:   #chr:+, st, end (0-base)
        ref_exon_starts[i[0]].add(int(i[1]))
        ref_exon_ends[i[0]].add(int(i[2]))
        if i[0] not in ref_exon_ranges:
            ref_exon_ranges[i[0]] = Intersecter()
        ref_exon_ranges[i[0]].add_interval( Interval( int(i[1]), int(i[2]) ))
        
    #Extract UTR exons from reference bed
    tmp = getUTRExonFromFile(refbed)
    for i in tmp:   #chr:+, st, end (0-base)
        ref_exon_starts[i[0]].add(int(i[1]))
        ref_exon_ends[i[0]].add(int(i[2]))
        if i[0] not in ref_exon_ranges:
            ref_exon_ranges[i[0]] = Intersecter()
        ref_exon_ranges[i[0]].add_interval( Interval( int(i[1]), int(i[2]) ))
    
    #prepare data structure 
    ref_exon_chain = getExonFromFile2(refbed)

    #read input bed
    for line in open(inputbed,'r'):
        if line.startswith('#'):continue
        if line.startswith('track'):continue
        if line.startswith('browser'):continue
        if not line.strip(): continue
        line = line.strip()
        fields=line.split()
        
        chrom = fields[0]
        strand = fields[5]
        tx_start = int(fields[1])
        tx_end = int(fields[2])
        key = chrom + ":" +strand
        if key in ref_exon_ranges:
            if  len(ref_exon_ranges[key].find(tx_start,tx_end))==0: #input gene does NOT overlap with any known exons
                print(line + '\t' + 'novel(intergenic)')
            else:
                input_exon_chain=getExonFromLine(line)
                #print line + '\t' + 'overlap'
                #utr_3_exons = getUTRExon(line,utr=3)
                #utr_5_exons = getUTRExon(line,utr=5)
                #cds_exons = getCDSExon(line)
        else:
            print(line + '\t' + 'unknownChrom')
        
        
        #for utr3 in utr_3_exons:
        #   (chrom, st, end) = (utr3[0], int(utr3[1]),int(utr3[2]))
        #   if chrom in ref_exon_ranges:        
        #       if len(ref_exon_ranges[chrom].find(st,end))>0 :     #input exon overlap with known exon     
        #   else:
    
    
    
    
    
    
    
    
    
    
    
    
    
    



#!/usr/bin/env python
'''
calculate fragment size for each gene/transcript. For each transcript/gene, it Will report:
1) # of fragment that was used.
2) mean of fragment size
3) median of fragment size
4) stdev of fragment size
'''

import pysam
import sys,os
from numpy import mean,median,std
from optparse import OptionParser

if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def overlap_length2(lst1, lst2):
	l = 0
	for x in lst1:
		for y in lst2:
			l += len(list(range(max(x[0], y[0]), min(x[-1], y[-1])+1)))
	return l
		
	
def fragment_size(bedfile, samfile,qcut=30,ncut=5):
	'''calculate the fragment size for each gene'''
	for line in open(bedfile,'r'):
		exon_range = []	
		if line.startswith(('#','track','browser')):continue
		fields = line.split()
		chrom = fields[0]
		tx_start = int( fields[1] )
		tx_end = int( fields[2] )
		geneName = fields[3]
		trand    = fields[5].replace(" ","_")
		exon_starts = list(map( int, fields[11].rstrip( ',\n' ).split( ',' ) ))
		exon_starts = list(map((lambda x: x + tx_start ), exon_starts))
		exon_ends = list(map( int, fields[10].rstrip( ',\n' ).split( ',' ) ))
		exon_ends = list(map((lambda x, y: x + y ), exon_starts, exon_ends))
		geneID = "\t".join([str(i) for i in (chrom, tx_start, tx_end, geneName)])
		
		for st,end in zip(exon_starts,exon_ends):
			exon_range.append([st+1,end+1])
			#exon_range.append([chrom, st,end])
		
		try:
			alignedReads = samfile.fetch(chrom,tx_start,tx_end)
		except:
			yield '\t'.join([str(i) for i in (geneID, 0,0,0)])
			continue
		
		frag_sizes = []
		for aligned_read in alignedReads:
			if not aligned_read.is_paired:				#skip single sequencing
				continue
			if aligned_read.is_read2:
				continue
			if aligned_read.mate_is_unmapped:
				continue
			if aligned_read.is_qcfail:continue               #skip low quanlity                                      
			if aligned_read.is_duplicate:continue           #skip duplicate read
			if aligned_read.is_secondary:continue           #skip non primary hit           
			if aligned_read.mapq < qcut:
					continue
			
			read_st = aligned_read.pos
			mate_st = aligned_read.pnext
			if read_st > mate_st:
				(read_st, mate_st) = (mate_st, read_st)
			if read_st < tx_start or mate_st > tx_end:
				continue
			read_len = aligned_read.qlen
			map_range = [[read_st+1, mate_st]]
			#map_range = [[chrom, read_st, mate_st]]
			frag_len = overlap_length2(exon_range, map_range) + read_len
			frag_sizes.append(frag_len)
		if len(frag_sizes) < ncut:
			yield '\t'.join([str(i) for i in (geneID, len(frag_sizes), 0,0,0)])
		else:
			yield '\t'.join([str(i) for i in (geneID, len(frag_sizes), mean(frag_sizes), median(frag_sizes), std(frag_sizes))])


def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input",action="store",type="string",dest="input_file",help="Input BAM file") 
	parser.add_option("-r","--refgene",action="store",type="string",dest="refgene_bed",help="Reference gene model in BED format. Must be strandard 12-column BED file. [required]")
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality (phred scaled) for an alignment to be called \"uniquely mapped\". default=%default")
	parser.add_option("-n","--frag-num",action="store",type="int",dest="fragment_num",default=3,help="Minimum number of fragment. default=%default")

	(options,args)=parser.parse_args()

	if not (options.input_file and options.refgene_bed):
		parser.print_help()
		sys.exit(0)
	if not os.path.exists(options.input_file + '.bai'):
		print("cannot find index file of input BAM file", file=sys.stderr)
		print(options.input_file + '.bai' + " does not exists", file=sys.stderr)
		sys.exit(0)	

	for file in (options.input_file, options.refgene_bed):
		if not os.path.exists(file):
			print(file + " does NOT exists" + '\n', file=sys.stderr)
			sys.exit(0)
	
	print('\t'.join([str(i) for i in ("chrom","tx_start", "tx_end","symbol","frag_count","frag_mean","frag_median","frag_std")]))
	for tmp in fragment_size(options.refgene_bed, pysam.Samfile(options.input_file), options.map_qual, options.fragment_num):
		print(tmp)

if __name__ == '__main__':
        main()












			
			



#!/usr/bin/env python
'''
Calculate the RNA-seq reads coverage over gene body.
This module uses bigwig file as input.
'''
import sys
from sys import version_info, stderr, exit
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

from os import path
from optparse import OptionParser
from collections import defaultdict
from subprocess import call
from numpy import nan_to_num
from pyBigWig import open as openBigWig
from qcmodule import mystat

__author__ = "Liguo Wang, Santiago Revale"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Testing"


def coverageGeneBody_bigwig(bigFile, refbed, outfile, gtype='png'):
    '''Calculate reads coverage over gene body, from 5'to 3'. each gene will be equally divided
    into 100 regsions. bigFile is bigwig format file'''

    if refbed is None:
        print ("You must specify a bed file representing gene model", file=sys.stderr)
        exit(0)

    bw = openBigWig(bigFile, 'r')
    # Get chromosomes present in the bigwig file 
    chroms = bw.chroms().keys()

    print ("calculating coverage over gene body ...", file=sys.stderr)
    coverage = defaultdict(int)
    flag=0
    gene_count = 0
    with open(refbed, 'r') as handle:
        # Loop through the genes in the BED file
        for line in handle:
            try:
                if line.startswith(('#','track','browser')):
                    continue

                # Parse fields from gene tabls
                fields = line.split()
                chrom     = fields[0]
                tx_start  = int( fields[1] )
                tx_end    = int( fields[2] )
                geneName  = fields[3]
                strand    = fields[5]

                # Skip chromosomes present in the bed file but not present in the bigwig file.
                # This could happen with PATCHES or Unplaced chromosomes.
                if chrom not in chroms:
                    continue

                exon_starts = list(map( int, fields[11].rstrip( ',\n' ).split( ',' ) ))
                exon_starts = list(map((lambda x: x + tx_start ), exon_starts))
                exon_ends = list(map( int, fields[10].rstrip( ',\n' ).split( ',' ) ))
                exon_ends = list(map((lambda x, y: x + y ), exon_starts, exon_ends))
            except:
                print ("[NOTE: input bed must be 12-column] skipped this line: " + line, end = '\n', file=sys.stderr)
                continue

            # Count gene if it was properly read
            gene_count += 1

            gene_all_base=[]
            mRNA_len =0
            flag=0
            for st, end in zip(exon_starts, exon_ends):
                gene_all_base.extend(range(st+1, end+1))    # 0-based coordinates on genome
                mRNA_len = len(gene_all_base)
                if mRNA_len < 100:
                    flag = 1
                    break
            if flag == 1:
                continue

            # Sort coordinates according to the strand
            if strand == '-':
                gene_all_base.sort(reverse=True)
            else:
                gene_all_base.sort(reverse=False)

            # Get 100 points from each gene's coordinates
            percentile_base = []
            percentile_base = mystat.percentile_list(gene_all_base)

            for i in range(0, len(percentile_base)):
                sig = bw.values(chrom,percentile_base[i]-1,percentile_base[i])
                coverage[i] += nan_to_num(sig[0])

            print (" \t%d genes finished\r" % gene_count, end = ' ', file=sys.stderr)

    # Close bigwig file
    bw.close()
    print ("\n",file=sys.stderr)

    x_coord=[]
    y_coord=[]
    with open(outfile + ".geneBodyCoverage.txt", 'w') as handle:
        handle.write("percentile\tcount\n")
        for i in coverage:
            x_coord.append(str(i))
            y_coord.append(str(coverage[i]))
            handle.write("%i\t%i\n" % (i, coverage[i]))

    with open(outfile + ".geneBodyCoverage_plot.r", 'w') as handle:
        handle.write("%s(\'%s\')\n" % (gtype, outfile + ".geneBodyCoverage." + gtype))
        handle.write("x=1:100\n")
        handle.write("y=c(%s)\n" % ','.join(y_coord))
        handle.write("plot(x, y/%s, xlab=\"percentile of gene body (5'->3')\", ylab='average wigsum', type='s')\n" % gene_count)
        handle.write("dev.off()\n")

def main():
    usage = "%prog [options]" + '\n' + __doc__ + "\n"
    parser = OptionParser(usage, version="%prog " + __version__)
    parser.add_option("-i", "--input-file", action="store", type="string", dest="input_file", help="Coverage signal file in bigwig format")
    parser.add_option("-r", "--refgene", action="store", type="string", dest="ref_gene_model", help="Reference gene model in bed format. [required]")
    parser.add_option("-o", "--out-prefix", action="store", type="string", dest="output_prefix", help="Prefix of output files(s). [required]")
    parser.add_option("-t", "--graph-type", action="store", type="string", dest="graph_type", default="pdf", help="Graphic file type in \"pdf\", \"jpeg\", \"bmp\", \"bmp\", \"tiff\" or \"png\".default=%default [optional]")
    (options, args) = parser.parse_args()

    gt = options.graph_type.lower()
    if gt not in ('pdf', 'png', 'bmp', 'jpeg', 'tiff'):
        print ("graphic file type must be 'pdf' or 'png'", end = '\n', file=sys.stderr)
        parser.print_help()
        exit(0)

    if not (options.output_prefix and options.input_file and options.ref_gene_model):
        parser.print_help()
        exit(0)

    if not path.exists(options.ref_gene_model):
        print ('\n\n' + options.ref_gene_model + " does NOT exists", end='\n', file=sys.stderr)
        #parser.print_help()
        exit(0)

    if path.exists(options.input_file):
        coverageGeneBody_bigwig(options.input_file, options.ref_gene_model, options.output_prefix, gtype=options.graph_type)
        try:
            call("Rscript " + options.output_prefix + '.geneBodyCoverage_plot.r', shell=True)
        except:
            print ( "Cannot generate plot from " + options.output_prefix + ".geneBodyCoverage_plot.r", end = '\n', file=sys.stderr)
            pass
    else:
        print ('\n\n' + options.input_file + " does NOT exists", end = '\n', file=sys.stderr)
        #parser.print_help()
        exit(0)

if __name__ == '__main__':
        main()   



#!/usr/bin/env python
'''-------------------------------------------------------------------------------------------------
Calculate distribution of reads' GC content
-------------------------------------------------------------------------------------------------'''

#import built-in modules
import os,sys

if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

from optparse import OptionParser
import warnings
import string
import collections
import math
from time import strftime
import subprocess


#import my own modules
from qcmodule import SAM
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def printlog (mesg):
	'''print progress into stderr and log file'''
	mesg="@ " + strftime("%Y-%m-%d %H:%M:%S") + ": " + mesg
	LOG=open('class.log','a')
	print(mesg, file=sys.stderr)
	print(mesg, file=LOG)


def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM or SAM format.")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files(s).")
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality (phred scaled) for an alignment to be called \"uniquely mapped\". default=%default")
	(options,args)=parser.parse_args()

	if not (options.output_prefix and options.input_file):
		parser.print_help()
		sys.exit(0)
	if os.path.exists(options.input_file):
		obj = SAM.ParseBAM(options.input_file)
		obj.readGC(outfile=options.output_prefix, q_cut = options.map_qual)	
		try:
			subprocess.call("Rscript " + options.output_prefix +  ".GC_plot.r",shell=True)
		except:
			pass
	else:
		print('\n\n' + options.input_file + " does NOT exists" + '\n', file=sys.stderr)
		#parser.print_help()
		sys.exit(0)



if __name__ == '__main__':
	main()



"""
Setup script for RSeQC -- Comprehensive QC package for RNA-seq data
"""

import sys, os, platform, glob
from distutils.core import setup
from setuptools import *

if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

def main():
	setup(  name = "RSeQC",
            version = "3.0.0",
            python_requires='>=3.5',
            packages = find_packages( 'lib' ),
            package_dir = { '': 'lib' },
            package_data = { '': ['*.ps'] },
            scripts = glob.glob( "scripts/*.py"),
            ext_modules = [],
            py_modules = [ 'psyco_full' ],
            test_suite = 'nose.collector',
            setup_requires = ['nose>=0.10.4','cython>=0.12'],
            author = "Liguo Wang",
			author_email ="wangliguo78@gmail.com",
			platforms = ['Linux','MacOS'],
			requires = ['cython (>=0.17)'],
			install_requires = ['cython>=0.17','pysam','bx-python','numpy', 'pyBigWig'], 
            description = "RNA-seq QC Package",
            url = "http://rseqc.sourceforge.net/",
            zip_safe = False,
            dependency_links = [],
			classifiers=[
              'Development Status :: 5 - Production/Stable',
              'Environment :: Console',
              'Intended Audience :: Science/Research',
              'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
              'Operating System :: MacOS :: MacOS X',
              'Operating System :: POSIX',
              'Programming Language :: Python',
              'Topic :: Scientific/Engineering :: Bio-Informatics',
              ],
			keywords='RNA-seq, QC',
        )


if __name__ == "__main__":
	main()



#!/usr/bin/env python
'''Normalize bigwig signal to fixed wigsum (equivelent to total reads). Output wiggle file'''
import os,sys
import collections
from operator import itemgetter
from itertools import groupby

if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	


from optparse import OptionParser

import pyBigWig
from qcmodule import BED
from qcmodule import twoList
import numpy
__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Testing"

def main():
	usage="%prog [options]"
	parser = OptionParser(usage,version="%prog " + __version__)
	
	parser.add_option("-i","--bwfile",action="store",type="string",dest="BigWig_File",help="Input BigWig file. [required]")
	parser.add_option("-o","--output",action="store",type="string",dest="output_wig",help="Output wig file. [required]")
	parser.add_option("-t","--wigsum",action="store",type="int",dest="total_wigsum",default=100000000,help="Specified wigsum. 100000000 equals to coverage of 1 million 100nt reads. default=%default  [optional]")
	parser.add_option("-r","--refgene",action="store",type="string",dest="refgene_bed",help="Reference gene model in bed format. [optional]")	
	parser.add_option("-c","--chunk",action="store",type="int",dest="chunk_size",default=500000,help="Chromosome chunk size. Each chomosome will be cut into samll chunks of this size. Decrease chunk size will save more RAM. default=%default (bp) [optional]")
	parser.add_option("-f","--format",action="store",type="string",dest="out_format",default="bgr",help="Output format. either \"wig\" or \"bgr\". \"bgr\" save disk space but make program slower. default=%default")
	(options,args)=parser.parse_args()
	
	if not (options.BigWig_File and options.output_wig):
		parser.print_help()
		sys.exit(0)

	OUT=open(options.output_wig,'w')
	bw = pyBigWig.open(options.BigWig_File)
	
	if bw.isBigWig():
		pass
	else:
		print("%s is not a bigwig file!" % options.BigWig_File,  file=sys.stderr)
		sys.exit(0)
	
	print("Get chromosome sizes from BigWig header ...", file=sys.stderr)
	chrom_sizes = {}
	for chr,size in bw.chroms().items():
		chrom_sizes[chr] = size
	
	exons=[]
	WIG_SUM=0.0
	if (options.refgene_bed):	
		print("Extract exons from " + options.refgene_bed, file=sys.stderr)
		obj = BED.ParseBED(options.refgene_bed)
		exons = obj.getExon()
		print("Merge overlapping exons ...", file=sys.stderr)
		exons = BED.unionBed3(exons)
		print("Calculate wigsum covered by " + options.refgene_bed + ' only', file=sys.stderr)
		for chrom,st,end in exons:
			if bw.stats(chrom, st, end)[0] is None:
				continue
			bw_signal = bw.values(chrom,st,end)
			tmp = numpy.nansum(bw_signal)			#nan will be ignored. but if all items are 'nan', the result summay is 'nan' NOT 0
			if numpy.isnan(tmp):continue	
			WIG_SUM += tmp
		print("Total wigsum is %.2f\n" % WIG_SUM, file=sys.stderr)
	else:
		print("Calculate wigsum from " + options.BigWig_File, file=sys.stderr)
		for chr_name, chr_size in list(chrom_sizes.items()):		#iterate each chrom
			if bw.stats(chr_name, 0, chr_size)[0] is None:
				print("Skip " + chr_name + "!", file=sys.stderr)
				continue

			print("Processing " + chr_name + " ...", file=sys.stderr)	
			for interval in BED.tillingBed(chrName = chr_name,chrSize = chr_size,stepSize = options.chunk_size):
				if bw.stats(interval[0],interval[1],interval[2])[0] is None:
					continue
				bw_signal  = bw.values(interval[0],interval[1],interval[2])
				tmp = numpy.nansum(bw_signal)
				if numpy.isnan(tmp):continue
				WIG_SUM += tmp
		print("\nTotal wigsum is %.2f\n" % WIG_SUM, file=sys.stderr)
	
	try:
		weight = options.total_wigsum/WIG_SUM
	except:
		"Error, WIG_SUM cannot be 0"
		sys.exit(1)

	#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	print("Normalizing bigwig file ...", file=sys.stderr)
	for chr_name, chr_size in list(chrom_sizes.items()):          #iterate each chrom
		
		if bw.stats(chr_name, 0, chr_size)[0] is None:
			print("Skip " + chr_name + "!", file=sys.stderr)
			continue
		
		if options.out_format.upper() == "WIG":
			print("Writing " + chr_name + " ...", file=sys.stderr)
			OUT.write('variableStep chrom='+chr_name+'\n')
			for interval in BED.tillingBed(chrName = chr_name,chrSize = chr_size,stepSize = options.chunk_size):
				coord = interval[1]
				bw_signal = bw.values(chr_name,interval[1],interval[2])
				tmp = numpy.nansum(bw_signal)
				if numpy.isnan(tmp):continue
				bw_signal = numpy.nan_to_num(bw_signal) * weight
				for v in bw_signal:
					coord +=1
					if v != 0: print("%d\t%.2f" % (coord,v), file=OUT)
		elif options.out_format.upper() == "BGR":
			print("Writing " + chr_name + " ...", file=sys.stderr)
			#OUT.write('variableStep chrom='+chr_name+'\n')
			for interval in BED.tillingBed(chrName = chr_name,chrSize = chr_size,stepSize = options.chunk_size):
				v2p = collections.defaultdict(list)     #value to position
				range2p={}      #coorindate range to value, bedgraph. #[start]=[len,value]
				coord = interval[1]
				bw_signal = bw.values(chr_name,interval[1],interval[2])
				tmp = numpy.nansum(bw_signal)
				if numpy.isnan(tmp):continue
				bw_signal = numpy.nan_to_num(bw_signal) * weight
				for v in bw_signal:
					coord +=1
					if v != 0: v2p[v].append(coord)
				for v in v2p:
					for k,g in groupby(enumerate(v2p[v]), lambda i_x:i_x[0]-i_x[1]):
						for l in [list(map(itemgetter(1), g))]:
							range2p[l[0]-1] = [len(l),v]
				for i in sorted(range2p):
					print(chr_name + '\t' + str(i) +'\t' + str(i + range2p[i][0]) + '\t' + str(range2p[i][1]), file=OUT)
		else:
			print("unknown output format", file=sys.stderr)
			sys.exit(1)
				
			
			
if __name__=='__main__':
	main()



'''manipulate ndarray list''' 

from itertools import starmap
from operator import mul,add,sub

__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"

                                                    

def check_list(v1,v2):
    '''check if the length of two list is same'''
    if v1.size != v2.size:
        raise ValueError("the lenght of both arrays must be the same")
    pass

def Add(v1,v2):
    '''add two list'''                                                                                 
    check_list(v1,v2)
    return v1.__add__(v2)

def Subtract(v1,v2):
    '''subtract v2 from v1'''                                                                                 
    check_list(v1,v2)
    return v1.__sub__(v2)

def Product(v1,v2):
    '''return product of two list'''                                                                                 
    check_list(v1,v2)
    return v1.__mul__(v2)

def Division(v1,v2):
    '''return divide v1 by v2. add 1 to both v1 and v2'''                                                                                 
    check_list(v1,v2)
    return (v1+1).__div__(v2+1)

def Average(v1,v2):
    '''return arithmetic mean of two list'''                                                                                 
    check_list(v1,v2)
    return v1.__add__(v2)/2

def geometricMean(v1,v2):
    '''return geometric mean of two list'''                                                                                 
    check_list(v1,v2)
    return (v1.__mul__(v2))**0.5

def Max(v1,v2):
    '''pairwise comparison two list. return  the max one between two paried number'''                                                                                 
    check_list(v1,v2)
    return map(max,zip(v1,v2))

def Min(v1,v2):
    '''pairwise comparison two list. return  the max one between two paried number'''                                                                                 
    check_list(v1,v2)
    return map(min,zip(v1,v2))
def euclidean_distance(v1,v2):
    '''return euclidean distance'''                                                                                 
    check_list(v1,v2)
    return (sum((v1.__sub__(v2))**2) / v1.size)**0.5


#!/usr/bin/env python
'''
Calculate the inner distance (insert size)  of RNA-seq fragments. 

               RNA fragment
 _________________||_________________
|                                    |
|                                    |
||||||||||------------------||||||||||
  read_1      insert_size     read_2

fragment size = read_1 + insert_size + read_2
'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

import re
import string
from optparse import OptionParser
import warnings
import string
import collections
import math
from time import strftime
import subprocess

#import third-party modules
from bx.bitset import *
from bx.bitset_builders import *
from bx.intervals import *

#import my own modules
from qcmodule import SAM
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def printlog (mesg):
	'''print progress into stderr and log file'''
	mesg="@ " + strftime("%Y-%m-%d %H:%M:%S") + ": " + mesg
	LOG=open('class.log','a')
	print(mesg, file=sys.stderr)
	print(mesg, file=LOG)


def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM or SAM format.")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files(s)")
	parser.add_option("-r","--refgene",action="store",type="string",dest="ref_gene",help="Reference gene model in BED format.")
	parser.add_option("-k","--sample-size",action="store",type="int",dest="sampleSize",default=1000000,help="Number of read-pairs used to estimate inner distance. default=%default")
	parser.add_option("-l","--lower-bound",action="store",type="int",dest="lower_bound_size",default=-250,help="Lower bound of inner distance (bp). This option is used for ploting histograme. default=%default")
	parser.add_option("-u","--upper-bound",action="store",type="int",dest="upper_bound_size",default=250,help="Upper bound of inner distance (bp). This option is used for plotting histogram. default=%default")
	parser.add_option("-s","--step",action="store",type="int",dest="step_size",default=5,help="Step size (bp) of histograme. This option is used for plotting histogram. default=%default")	
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality (phred scaled) for an alignment to be called \"uniquely mapped\". default=%default")

	(options,args)=parser.parse_args()

	if not (options.output_prefix and options.input_file and options.ref_gene):
		parser.print_help()
		sys.exit(0)
	for input_file in ([options.input_file,options.ref_gene]):
		if not os.path.exists(input_file):
			print('\n\n' + input_file + " does NOT exists" + '\n', file=sys.stderr)
			parser.print_help()
			sys.exit(0)
	if options.step_size <=0:
		print("step size is a positive interger", file=sys.stderr)
		sys.exit(0)
	obj = SAM.ParseBAM(options.input_file)
	obj.mRNA_inner_distance(outfile=options.output_prefix,low_bound=options.lower_bound_size,up_bound=options.upper_bound_size,step=options.step_size,refbed=options.ref_gene,sample_size=options.sampleSize, q_cut = options.map_qual)
	try:
		subprocess.call("Rscript " + options.output_prefix + '.inner_distance_plot.r',shell=True)
	except:
		print("Cannot generate pdf file from " + options.output_prefix + '.inner_distance_plot.r', file=sys.stderr)
		pass

if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''
Convert BAM file into wig file. BAM file must be sorted and indexed using SAMtools.
Note: SAM format file is not supported.
'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

import string
from optparse import OptionParser
import warnings
import string
import collections

#import third-party modules
from bx.bitset import *
from bx.bitset_builders import *
from bx.intervals import *

#import my own modules
from qcmodule import SAM
from qcmodule import BED
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def printlog (mesg):
	'''print progress into stderr and log file'''
	mesg="@ " + strftime("%Y-%m-%d %H:%M:%S") + ": " + mesg
	LOG=open('class.log','a')
	print(mesg, file=sys.stderr)
	print(mesg, file=LOG)

def load_chromsize(file):
	'''read chrom.size file'''
	chromSize={}
	for line in open(file,'r'):
		if line.startswith('#'):continue
		if not line.strip():continue
		fields = line.strip().split()
		chromSize[fields[0]] = int(fields[1])
	return chromSize

			
def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM format. BAM file must be sorted and indexed using samTools. .bam and .bai files should be placed in the same directory.")
	parser.add_option("-s","--chromSize",action="store",type="string",dest="chromSize",help="Chromosome size file. Tab or space separated text file with 2 columns: first column is chromosome name/ID, second column is chromosome size. Chromosome name (such as \"chr1\") should be consistent between this file and the BAM file.")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output wiggle files(s). One wiggle file will be generated for non strand-specific data, two wiggle files (\"Prefix_Forward.wig\" and \"Prefix_Reverse.wig\") will be generated for strand-specific RNA-seq data.")
	parser.add_option("-t","--wigsum",action="store",type="int",dest="total_wigsum",help="Specified wigsum. Eg: 1,000,000,000 equals to coverage of 10 million 100nt reads. Ignore this option to disable normalization")
	parser.add_option("-u","--skip-multi-hits",action="store_true",dest="skip_multi",help="Skip non-unique hit reads.")
	parser.add_option("-d","--strand",action="store",type="string",dest="strand_rule",default=None,help="How read(s) were stranded during sequencing. For example: --strand='1++,1--,2+-,2-+' means that this is a pair-end, strand-specific RNA-seq data, and the strand rule is: read1 mapped to '+' => parental gene on '+'; read1 mapped to '-' => parental gene on '-'; read2 mapped to '+' => parental gene on '-'; read2 mapped to '-' => parental gene on '+'.  If you are not sure about the strand rule, run \'infer_experiment.py' default=%default (Not a strand specific RNA-seq data).")
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality to determine \"uniquely mapped\". default=%default")

	(options,args)=parser.parse_args()
	
	
	if not (options.output_prefix and options.input_file and options.chromSize and options.output_prefix):
		parser.print_help()
		sys.exit(0)
	for file in (options.input_file,options.chromSize):
		if not os.path.exists(file):
			print('\n\n' + file + " does NOT exists" + '\n', file=sys.stderr)
			sys.exit(0)
	if not os.path.exists(options.input_file + '.bai'):
		print("index file " + options.input_file + '.bai' + " does not exists", file=sys.stderr)
		sys.exit(0)
	
	if options.skip_multi:print("Skip multi-hits:True")
	else:print("Skip multi-hits:False")


	chromSizes = load_chromsize(options.chromSize)
	
	norm_factor=None
	if options.total_wigsum:
		obj = SAM.ParseBAM(options.input_file)
		wig_sum = obj.calWigSum(chrom_sizes = chromSizes, skip_multi=options.skip_multi)
		print("\n\ntotal wigsum is:" + str(wig_sum) + '\n', file=sys.stderr)
		try:
			norm_factor = options.total_wigsum / wig_sum
		except:
			norm_factor = None
			
	obj = SAM.ParseBAM(options.input_file)		
	obj.bamTowig(outfile = options.output_prefix, chrom_sizes = chromSizes, chrom_file = options.chromSize, q_cut = options.map_qual, skip_multi=options.skip_multi,strand_rule = options.strand_rule, WigSumFactor=norm_factor)

if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''
Calculate the distributions of deleted nucleotides across reads. 
'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

import string
import subprocess
from optparse import OptionParser
from time import strftime


#import my own modules
from qcmodule import SAM
from qcmodule import fasta
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def printlog (mesg):
	'''print progress into stderr and log file'''
	mesg="@ " + strftime("%Y-%m-%d %H:%M:%S") + ": " + mesg
	print(mesg, file=sys.stderr)


def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input",action="store",type="string",dest="input_bam",help='Input BAM file. [required]')
	parser.add_option("-l","--read-align-length",action="store",type="int", dest="read_alignment_length",help="Alignment length of read. It is usually set to the orignial read length. For example, all these cigar strings (\"101M\", \"68M140N33M\", \"53M1D48M\") suggest the read alignment length is 101. [required]")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files(s). [required]")
	parser.add_option("-n","--read-num",action="store",type="int",default=1000000, dest="read_number",help="Number of aligned reads with deletions used to calculate the deletion profile. default=%default")
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality. default=%default")
	(options,args)=parser.parse_args()

	if not (options.input_bam):
		parser.print_help()
		sys.exit(0)
	for f in ([options.input_bam]):
		if not os.path.exists(f):
			print('\n\n' + f + " does NOT exists" + '\n', file=sys.stderr)
			parser.print_help()
			sys.exit(0)

	if not (options.output_prefix):
		print('\n\n You must specify the output prefix', file=sys.stderr)
		parser.print_help()
		sys.exit(0)
	
	if not (options.read_alignment_length):
		print('\n\n You must specify read alignment length. It is usually the read length.', file=sys.stderr)
		parser.print_help()
		sys.exit(0)	
		
	obj = SAM.ParseBAM(options.input_bam)
	obj.deletionProfile(read_length = options.read_alignment_length, read_num = options.read_number, q_cut = options.map_qual, outfile = options.output_prefix)
	
	try:
		subprocess.call("Rscript " + options.output_prefix + '.deletion_profile.r',shell=True)
	except:
		print("Cannot generate pdf file from " + options.output_prefix + '.deletion_profile.r', file=sys.stderr)
		pass
	
if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''
This program is to estimate clipping profile of RNA-seq reads from BAM or SAM file
Note that to use this funciton, CIGAR strings within SAM/BAM file should have 'S' operation 
(This means your reads mapper should support clipped mapping).
'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

import re
import string
from optparse import OptionParser
import warnings
import string
import collections
import math
from time import strftime
import subprocess
 

#import third-party modules
from bx.bitset import *
from bx.bitset_builders import *
from bx.intervals import *

#import my own modules
from qcmodule import SAM
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM or SAM format.")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files(s).")
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality (phred scaled) for an alignment to be considered as \"uniquely mapped\". default=%default")
	parser.add_option("-s","--sequencing",action="store",dest="layout",help="Sequencing layout. \"SE\"(single-end) or \"PE\"(pair-end). ")
	(options,args)=parser.parse_args()

	if not (options.input_file and options.output_prefix and options.layout):
		parser.print_help()
		sys.exit(0)
	for input_file in ([options.input_file]):
		if not os.path.exists(input_file):
			print('\n\n' + input_file + " does NOT exists" + '\n', file=sys.stderr)
			sys.exit(0)

	obj = SAM.ParseBAM(options.input_file)
	if options.layout == "SE":
		obj.clipping_profile(outfile=options.output_prefix, q_cut = options.map_qual,type="S", PE=False)
	elif options.layout == "PE":
		obj.clipping_profile(outfile=options.output_prefix, q_cut = options.map_qual, type="S", PE=True)
	else:
		print("unknow sequencing layout. Must be \"SE\" or \"PE\"", file=sys.stderr)
	try:
		subprocess.call("Rscript " + options.output_prefix + '.clipping_profile.r',shell=True)
	except:
		print("Cannot generate pdf file from " + options.output_prefix + '.clipping_profile.r', file=sys.stderr)
		pass

if __name__ == '__main__':
	main()
